"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventBus = void 0;
const cqrs_1 = require("@nestjs/cqrs");
const common_1 = require("@nestjs/common");
const core_1 = require("@nestjs/core");
const command_bus_1 = require("@nestjs/cqrs/dist/command-bus");
const constants_1 = require("@nestjs/cqrs/dist/decorators/constants");
const decorators_1 = require("./decorators");
const services_1 = require("./services");
let EventBus = class EventBus extends cqrs_1.EventBus {
    constructor(_commandBus, _moduleRef, _nmc) {
        super(_commandBus, _moduleRef);
        this._commandBus = _commandBus;
        this._moduleRef = _moduleRef;
        this._nmc = _nmc;
    }
    isLocal(event) {
        return Reflect.getMetadata(decorators_1.LOCAL_EVENT_METADATA, event.constructor);
    }
    splitEventsListLocalGlobal(events) {
        const local = [];
        const global = [];
        const that = this;
        events.forEach(event => {
            if (that.isLocal(event)) {
                local.push(event);
            }
            else {
                global.push(event);
            }
        });
        return { local, global };
    }
    publish(event) {
        if (this.isLocal(event)) {
            return this.publishLocally(event);
        }
        else {
            return this.emit(event);
        }
    }
    publishLocally(event) {
        return super.publish(event);
    }
    publishAll(events) {
        const eventSplit = this.splitEventsListLocalGlobal(events);
        const that = this;
        eventSplit.global.forEach((event) => {
            that.emit(event);
        });
        events = eventSplit.local;
        return this.publishAllLocally(events);
    }
    publishAllLocally(events) {
        return super.publishAll(events);
    }
    emit(event) {
        return this._nmc.emit(event.constructor.name, event.json());
    }
    registerHandler(handler) {
        const instance = this._moduleRef.get(handler, { strict: false });
        if (!instance) {
            return;
        }
        const eventsNames = this._reflectEventsNames(handler);
        eventsNames.forEach((event) => {
            if (!this._reflectGlobalEvent(event)) {
                this.subscribeGlobally(event);
            }
        });
        eventsNames.map((event) => this.bind(instance, event.name));
    }
    subscribeGlobally(event) {
        const that = this;
        this._nmc.addHandler(event.name, (name, data) => {
            const instance = new event();
            instance.fromObj(data);
            that.publishLocally(instance);
        });
    }
    _reflectGlobalEvent(event) {
        return Reflect.getMetadata(decorators_1.LOCAL_EVENT_METADATA, event);
    }
    _reflectEventsNames(handler) {
        return Reflect.getMetadata(constants_1.EVENTS_HANDLER_METADATA, handler);
    }
};
EventBus = __decorate([
    common_1.Injectable(),
    __metadata("design:paramtypes", [command_bus_1.CommandBus,
        core_1.ModuleRef,
        services_1.NestjsMicroservicesCqrs])
], EventBus);
exports.EventBus = EventBus;
